searchNodes=[{"doc":"Vela is a tiny library providing easy management of validated cached state with some history. Including use Vela in your module would turn the module into struct, setting field accordingly to the specification, passed as a parameter. Vela allows the following configurable parameters per field: limit  — length of the series to keep (default: 5 ) compare_by  — comparator extraction function to extract the value, to be used for comparison, from the underlying terms (default: &amp; &amp;1 ) comparator — the function that accepts a series name and two values and returns the greater one to be used in Vela.δ/1 (default: &amp;&lt;/2 ) threshold  — if specified, the inserted value is checked to fit in δ ± threshold ; whether it does not, it goes to errors ( float() | nil , default: nil ) validator  — the function to be used to invalidate the accumulated values (default: fn _ -&gt; true end ) sorter — the function to be used to sort values within one serie, if none is given, it sorts in the natural order, FIFO, newest is the one to pop corrector — the function to be used to correct the values rejected by validator ; the function should return {:ok, corrected_value} to enforce insertion into Vela , or :error if the value cannot be corrected and should be nevertheless rejected errors  — number of errors to keep (default: 5 ) Also, Vela accepts :__meta__ keyword parameter for the cases when the consumer needs the very custom meta to be passed to the struct. Vela implements Access behaviour. Usage defmodule Vela.Test do use Vela , series1 : [ limit : 3 , errors : 1 ] , # no validation series2 : [ limit : 2 , validator : Vela.Test ] series3 : [ compare_by : &amp; Vela.Test . comparator / 1 , validator : &amp; Vela.Test . validator / 2 ] @behaviour Vela.Validator @impl Vela.Validator def valid? ( _serie , value ) , do : value &gt; 0 @spec comparator ( %{ created_at :: DateTime . t ( ) } ) :: DateTime . t ( ) def comparator ( %{ created_at : created_at } ) , do : created_at @spec validator ( value :: t ( ) ) :: boolean ( ) def validator ( value ) , do : is_integer ( value ) and value &gt; 300 end In the example above, before any structure update attempt (via Access ,) this valid?/2 function would be called. If it returns true , the value gets inserted / updated, and the series behind is truncated if needed. It it returns false , the state is not updated , and the value is put into the map under __errors__ key of the struct. The length of errors is also configurable via errors: keyword parameter.","ref":"Vela.html","title":"Vela","type":"behaviour"},{"doc":"Returns a keyword with series as keys and the average value as a value Example: iex&gt; defmodule XY do ...&gt; use Vela , x : [ ] , y : [ ] ...&gt; end ...&gt; XY . average ( struct ( XY , x : [ 1 , 2 , 3 ] , y : [ 5 , 7 , 9 ] ) , &amp; ( Enum . sum ( &amp;1 ) / length ( &amp;1 ) ) ) [ x : 2.0 , y : 7.0 ] ...&gt; defmodule Averager do ...&gt; def average ( values ) , do : Enum . sum ( values ) / length ( values ) ...&gt; end ...&gt; XY . average ( struct ( XY , x : [ 1 , 2 , 3 ] , y : [ 5 , 7 , 9 ] ) , Averager ) [ x : 2.0 , y : 7.0 ]","ref":"Vela.html#c:average/2","title":"Vela.average/2","type":"callback"},{"doc":"Returns {min, max} tuple for each serie, using the comparator given as a second parameter, or a default comparator for this serie.","ref":"Vela.html#c:delta/2","title":"Vela.delta/2","type":"callback"},{"doc":"Empties the Vela given.","ref":"Vela.html#empty!/1","title":"Vela.empty!/1","type":"function"},{"doc":"Returns true if the Vela is empty, false otherwise.","ref":"Vela.html#empty?/1","title":"Vela.empty?/1","type":"function"},{"doc":"Checks two velas given as an input for equality","ref":"Vela.html#c:equal?/2","title":"Vela.equal?/2","type":"callback"},{"doc":"Flat maps the series using fun and returns the keyword with duplicated keys and mapped values. Example: defmodule EO do use Vela , even : [ limit : 2 ] , odd : [ limit : 2 ] def flat_map ( % EO { } = v ) , do : Vela . flat_map ( v , &amp; { &amp;1 , &amp;2 + 1 } ) end EO . flat_map ( struct ( EO , [ even : [ 2 , 4 ] , odd : [ 1 , 3 ] ] ) ) #⇒ [even: 3, even: 5, odd: 2, odd: 4]","ref":"Vela.html#flat_map/2","title":"Vela.flat_map/2","type":"function"},{"doc":"Maps the series using fun and returns the new Vela instance with series mapped","ref":"Vela.html#map/2","title":"Vela.map/2","type":"function"},{"doc":"Merges two Vela s given using resolver/3 function. This function does not allow merging metas, the first argument wins.","ref":"Vela.html#merge/3","title":"Vela.merge/3","type":"function"},{"doc":"Removes obsoleted elements from the series using the validator given as a second parameter, or a default validator for this serie.","ref":"Vela.html#c:purge/2","title":"Vela.purge/2","type":"callback"},{"doc":"Inserts the new value into the serie, going through all the validation and sorting. If the value has not passed validation, it’s put into :__errors__ internal list. If the new length of the serie exceeds the limit set for this serie, the last value (after sorting) gets discarded.","ref":"Vela.html#put/3","title":"Vela.put/3","type":"function"},{"doc":"Returns a keyword with series as keys and the hottest value as a value Example: iex&gt; defmodule AB do ...&gt; use Vela , a : [ ] , b : [ ] , c : [ ] ...&gt; end ...&gt; AB . slice ( struct ( AB , [ a : [ 1 , 2 ] , b : [ 3 ] , c : [ ] ] ) ) [ a : 1 , b : 3 ]","ref":"Vela.html#c:slice/1","title":"Vela.slice/1","type":"callback"},{"doc":"Returns {min, max} tuple for each serie, using the comparator given as a second parameter, or a default comparator for each serie.","ref":"Vela.html#%CE%B4/2","title":"Vela.δ/2","type":"function"},{"doc":"The type of comparator function to be passed as :comparator keyword parameter to the series.","ref":"Vela.html#t:comparator/0","title":"Vela.comparator/0","type":"type"},{"doc":"The type of sorter function to be passed as :sorter keyword parameter to the series.","ref":"Vela.html#t:corrector/0","title":"Vela.corrector/0","type":"type"},{"doc":"Represents a key-value pair in errors and unmatched","ref":"Vela.html#t:kv/0","title":"Vela.kv/0","type":"type"},{"doc":"Options allowed in series configuration","ref":"Vela.html#t:option/0","title":"Vela.option/0","type":"type"},{"doc":"Series configuration","ref":"Vela.html#t:options/0","title":"Vela.options/0","type":"type"},{"doc":"Represents a key in the Vela structure","ref":"Vela.html#t:serie/0","title":"Vela.serie/0","type":"type"},{"doc":"The type of sorter function to be passed as :sorter keyword parameter to the series.","ref":"Vela.html#t:sorter/0","title":"Vela.sorter/0","type":"type"},{"doc":"Represents the internal state aka per-vela propery container","ref":"Vela.html#t:state/0","title":"Vela.state/0","type":"type"},{"doc":"Represents the struct created by this behaviour module","ref":"Vela.html#t:t/0","title":"Vela.t/0","type":"type"},{"doc":"The type of validator function to be passed as :validator keyword parameter to the series.","ref":"Vela.html#t:validator/0","title":"Vela.validator/0","type":"type"},{"doc":"Represents a value in the Vela structure","ref":"Vela.html#t:value/0","title":"Vela.value/0","type":"type"},{"doc":"The validator behaviour, implementations are used by Vela to validate new values to be inserted into the state.","ref":"Vela.Validator.html","title":"Vela.Validator","type":"behaviour"},{"doc":"When implemented, the module might be passed as validator to use Vela","ref":"Vela.Validator.html#c:valid?/2","title":"Vela.Validator.valid?/2","type":"callback"},{"doc":"The generic exception thrown when the access in invalid. Parameters: source the source of the exception field the field the access to has failed","ref":"Vela.AccessError.html","title":"Vela.AccessError","type":"exception"},{"doc":"The tiny library to ease handling expiring invalidated cached series","ref":"readme.html","title":"Vela","type":"extras"},{"doc":"def deps do [ { :vela , &quot;~&gt; 0.12&quot; } ] end","ref":"readme.html#installation","title":"Vela - Installation","type":"extras"},{"doc":"0.16.1 — Vela.merge/3 0.16.0 — Vela.empty?/1 , Vela.empty!/1 and some performance improvements in Vela.equal?/2 0.15.3 — allow all metas to reside in state 0.15.2 — less restrictive slice/2 , accepting malformed velas 0.15.0 — :mη keyword parameter in a call to use Vela has been renamed to :__meta__ as Elixir v1.14 does not support mixed scripts (meh) 0.14.0 — average/2 to produce a keyword of {serie, average} values 0.13.1 — state/1 and update_state/2 to keep some additional date alongside with Vela 0.12.0 — threshold does not depend on the band anymore 0.11.0 — __meta__ might be now used to overwrite compiled in serie settings 0.9.5  — early return the value provided by existing comparison function in equal?/2 0.9.3  — allow :atom and {GenServer, :on_start} as type definition 0.9.0  — allow a precise type definition of each serie via type: type() keyword parameter a series keyword parameter 0.8.0  — allow a corrector/2 callback to allow correction of rejected values as a series keyword parameter 0.7.2  —  Vela.put/3 0.7.0  — exact type and behaviour for those using Vela 0.6.3  — fix threshold to use compare_by/1 for cumbersome values 0.6.1  — use threshold to prevent adding outliers to series 0.6.0  —  Vela.δ/2 / Vela.delta/2 returning a keyword [{serie, {min, max}}]","ref":"readme.html#changelog","title":"Vela - Changelog","type":"extras"},{"doc":"","ref":"readme.html#documentation","title":"Vela - Documentation","type":"extras"},{"doc":"Vela is designed to collect and keep a set of series of different values. Each serie is supposed to keep validated , ordered , actual values. validation is being done upon insertion, using validator parameter for this particular serie ( default: always allow;) if the validation fails, the value gets discarded immediately obsoleted values are removed by explicit call to purge/2 sorting is done with sorter / compare_by pair of parameters; default sorting is none ( LIFO ) and the default comparator is itself . Consider the following example. Let’s declare Vela having two series, one with integers, another with dates. We want both to be sorted descending so that we could get the most actual values at any time. defmodule Checkers do def good_integer ( :integers , int ) when is_integer ( int ) , do : true def good_integer ( _ , _ ) , do : false def good_date ( :dates , % Date { } ) , do : true def good_date ( _ , _ ) , do : false def compare_dates ( % Date { } = d1 , % Date { } = d2 ) , do : Date . compare ( d1 , d2 ) == :gt end defmodule Series do import Checkers use Vela , integers : [ limit : 3 , validator : &amp; good_integer / 2 , sorter : &amp; &gt;= / 2 ] , dates : [ limit : 3 , validator : &amp; good_date / 2 , sorter : &amp; compare_dates / 2 ] end Now we can fill the struct with values. iex | 1 &gt; s = % Series { } #⇒ %Series{__errors__: [], dates: [], integers: []} iex | 2 &gt; put_in s , [ :integers ] , 42 #⇒ %Series{__errors__: [], dates: [], integers: &#39;*&#39;} iex | 3 &gt; put_in s , [ :integers ] , &quot;42&quot; #⇒ %Series{__errors__: [integers: &quot;42&quot;], dates: [], integers: []} iex | 4 &gt; put_in s , [ :dates ] , Date . utc_today #⇒ %Series{__errors__: [], dates: [~D[2020-06-30]], integers: []} iex | 5 &gt; put_in s , [ :dates ] , &quot;42&quot; #⇒ %Series{__errors__: [dates: &quot;42&quot;], dates: [], integers: []} Let’s see how sorting works. iex | 6 &gt; s ... | 6 &gt; |&gt; put_in ( [ :integers ] , 10 ) ... | 6 &gt; |&gt; put_in ( [ :integers ] , 20 ) ... | 6 &gt; |&gt; put_in ( [ :integers ] , 30 ) ... | 6 &gt; |&gt; put_in ( [ :integers ] , 40 ) ... | 6 &gt; |&gt; put_in ( [ :integers ] , 0 ) % Series { __errors__ : [ ] , dates : [ ] , integers : [ 40 , 30 , 20 ] } We instructed the serie to keep at most three values, sorted descending. The get to the most “relevant” values (heads of series,) one might use slice/1 . It returns the keyword with all the actual values. iex | 7 &gt; Series . slice s #⇒ [integers: 40] To get to the head of each serie, one might use Access as shown below. iex | 8 &gt; get_in s , [ :integers ] #⇒ 40","ref":"lifetime.html","title":"Vela Lifetime","type":"extras"}]